@@ -10,34 +10,42 @@ from cryspy.B_parent_classes.cl_2_loop import LoopN
 from cryspy.B_parent_classes.cl_3_data import DataN
 from cryspy.B_parent_classes.cl_4_global import GlobalN
 
-from cryspy.C_item_loop_classes.cl_1_cell import Cell, CellL
 from cryspy.C_item_loop_classes.cl_1_atom_site import AtomSite, \
     AtomSiteL
 from cryspy.C_item_loop_classes.cl_1_atom_type import AtomType, \
     AtomTypeL
 from cryspy.C_item_loop_classes.cl_1_atom_site_aniso import \
     AtomSiteAniso, AtomSiteAnisoL
-from cryspy.C_item_loop_classes.cl_1_refln import Refln, ReflnL
+from cryspy.C_item_loop_classes.cl_1_atom_site_moment import \
+    AtomSiteMoment, AtomSiteMomentL
 from cryspy.C_item_loop_classes.cl_1_atom_site_susceptibility import \
     AtomSiteSusceptibility, AtomSiteSusceptibilityL
 from cryspy.C_item_loop_classes.cl_1_atom_type_scat import \
     AtomTypeScat, AtomTypeScatL
-from cryspy.C_item_loop_classes.cl_1_refln_susceptibility import \
-    ReflnSusceptibility, ReflnSusceptibilityL
 from cryspy.C_item_loop_classes.cl_1_atom_local_axes import \
     AtomLocalAxes, AtomLocalAxesL
 from cryspy.C_item_loop_classes.cl_1_atom_electron_configuration \
     import AtomElectronConfiguration, AtomElectronConfigurationL
-from cryspy.C_item_loop_classes.cl_1_setup import Setup, SetupL
+from cryspy.C_item_loop_classes.cl_1_cell import Cell, CellL
+from cryspy.C_item_loop_classes.cl_1_chi2 import Chi2, Chi2L
 from cryspy.C_item_loop_classes.cl_1_diffrn_radiation import \
     DiffrnRadiation, DiffrnRadiationL
 from cryspy.C_item_loop_classes.cl_1_diffrn_refln import DiffrnRefln, \
     DiffrnReflnL
+from cryspy.C_item_loop_classes.cl_1_exclude import Exclude, ExcludeL
 from cryspy.C_item_loop_classes.cl_1_extinction import Extinction, \
     ExtinctionL
 from cryspy.C_item_loop_classes.cl_1_phase import Phase, PhaseL
-from cryspy.C_item_loop_classes.cl_1_refine_ls import RefineLs, \
-    RefineLsL
+from cryspy.C_item_loop_classes.cl_1_pd2d_instr_reflex_asymmetry import\
+    Pd2dInstrReflexAsymmetry, Pd2dInstrReflexAsymmetryL
+from cryspy.C_item_loop_classes.cl_1_pd2d_instr_resolution import \
+    Pd2dInstrResolution, Pd2dInstrResolutionL
+from cryspy.C_item_loop_classes.cl_1_pd2d_meas import Pd2dMeas
+from cryspy.C_item_loop_classes.cl_1_pd2d_proc import Pd2dProc
+from cryspy.C_item_loop_classes.cl_1_pd2d_peak import Pd2dPeak, \
+    Pd2dPeakL
+from cryspy.C_item_loop_classes.cl_1_pd2d_background import \
+    Pd2dBackground
 from cryspy.C_item_loop_classes.cl_1_pd_background import \
     PdBackground, PdBackgroundL
 from cryspy.C_item_loop_classes.cl_1_pd_instr_reflex_asymmetry import \
@@ -47,34 +55,33 @@ from cryspy.C_item_loop_classes.cl_1_pd_instr_resolution import \
 from cryspy.C_item_loop_classes.cl_1_pd_meas import PdMeas, PdMeasL
 from cryspy.C_item_loop_classes.cl_1_pd_proc import PdProc, PdProcL
 from cryspy.C_item_loop_classes.cl_1_pd_peak import PdPeak, PdPeakL
-from cryspy.C_item_loop_classes.cl_1_chi2 import Chi2, Chi2L
 from cryspy.C_item_loop_classes.cl_1_range import Range, RangeL
+from cryspy.C_item_loop_classes.cl_1_refln import Refln, ReflnL
+from cryspy.C_item_loop_classes.cl_1_refln_susceptibility import \
+    ReflnSusceptibility, ReflnSusceptibilityL
+from cryspy.C_item_loop_classes.cl_1_refine_ls import RefineLs, \
+    RefineLsL
+from cryspy.C_item_loop_classes.cl_1_setup import Setup, SetupL
+from cryspy.C_item_loop_classes.cl_1_space_group_symop_magn_centering import \
+    SpaceGroupSymopMagnCentering, SpaceGroupSymopMagnCenteringL
 from cryspy.C_item_loop_classes.cl_1_texture import Texture, TextureL
-from cryspy.C_item_loop_classes.cl_1_exclude import Exclude, ExcludeL
-from cryspy.C_item_loop_classes.cl_1_pd2d_background import \
-    Pd2dBackground
-from cryspy.C_item_loop_classes.cl_1_pd2d_instr_reflex_asymmetry import\
-    Pd2dInstrReflexAsymmetry, Pd2dInstrReflexAsymmetryL
-from cryspy.C_item_loop_classes.cl_1_pd2d_instr_resolution import \
-    Pd2dInstrResolution, Pd2dInstrResolutionL
-from cryspy.C_item_loop_classes.cl_1_pd2d_meas import Pd2dMeas
-from cryspy.C_item_loop_classes.cl_1_pd2d_proc import Pd2dProc
-from cryspy.C_item_loop_classes.cl_1_pd2d_peak import Pd2dPeak, \
-    Pd2dPeakL
 from cryspy.C_item_loop_classes.cl_1_mem_parameters import \
     MEMParameters, MEMParametersL
 
-from cryspy.C_item_loop_classes.cl_2_space_group import SpaceGroup
 from cryspy.C_item_loop_classes.cl_2_atom_site_scat import \
     AtomSiteScat, AtomSiteScatL
 from cryspy.C_item_loop_classes.cl_2_diffrn_orient_matrix import \
     DiffrnOrientMatrix, DiffrnOrientMatrixL
 from cryspy.C_item_loop_classes.cl_2_section import Section, SectionL
+from cryspy.C_item_loop_classes.cl_2_space_group import SpaceGroup
+from cryspy.C_item_loop_classes.cl_2_space_group_symop_magn_operation import \
+    SpaceGroupSymopMagnOperation, SpaceGroupSymopMagnOperationL
 
 from cryspy.C_item_loop_classes.cl_3_density_point import DensityPoint, \
     DensityPointL
 
 from cryspy.E_data_classes.cl_1_crystal import Crystal
+from cryspy.E_data_classes.cl_1_mag_crystal import MagCrystal
 from cryspy.E_data_classes.cl_2_diffrn import Diffrn
 from cryspy.E_data_classes.cl_2_pd import Pd
 from cryspy.E_data_classes.cl_2_pd2d import Pd2d
@@ -90,26 +97,35 @@ L_GLOBAL_CLASS = []
 L_FUNCTION_ALL = []
 
 L_ITEM_CLASS.extend([
-    SpaceGroup, Cell, AtomSite, AtomType, AtomSiteAniso, Refln,
-    AtomSiteSusceptibility, AtomTypeScat, AtomSiteScat, ReflnSusceptibility,
-    AtomLocalAxes, AtomElectronConfiguration, Setup, DiffrnRadiation,
-    DiffrnOrientMatrix, DiffrnRefln, Extinction, Phase, RefineLs, PdBackground,
-    PdInstrReflexAsymmetry, PdInstrResolution, PdMeas, PdProc, PdPeak, Chi2,
-    Range, Texture, Exclude, Pd2dBackground, Pd2dInstrReflexAsymmetry,
-    Pd2dInstrResolution, Pd2dMeas, Pd2dProc, Pd2dPeak, DensityPoint,
-    MEMParameters, Section])
+    AtomElectronConfiguration, AtomLocalAxes,
+    AtomSite, AtomSiteAniso, AtomSiteMoment, AtomSiteScat,
+    AtomSiteSusceptibility, AtomType, AtomTypeScat,
+    Cell, Chi2,
+    DensityPoint, DiffrnRadiation, DiffrnOrientMatrix, DiffrnRefln,
+    Exclude, Extinction, MEMParameters,
+    Phase, PdBackground,
+    PdInstrReflexAsymmetry, PdInstrResolution, PdMeas, PdProc, PdPeak,
+    Pd2dBackground, Pd2dInstrReflexAsymmetry,
+    Pd2dInstrResolution, Pd2dMeas, Pd2dProc, Pd2dPeak,
+    Range, RefineLs, Refln, ReflnSusceptibility,
+    Setup, SpaceGroupSymopMagnCentering, SpaceGroupSymopMagnOperation,
+    SpaceGroup, Section, Texture])
 
 L_LOOP_CLASS.extend([
-    CellL, AtomSiteL, AtomTypeL, AtomSiteAnisoL, ReflnL,
-    AtomSiteSusceptibilityL, AtomTypeScatL, AtomSiteScatL,
-    ReflnSusceptibilityL, AtomLocalAxesL, AtomElectronConfigurationL, SetupL,
-    DiffrnRadiationL, DiffrnOrientMatrixL, DiffrnReflnL,  ExtinctionL, PhaseL,
-    RefineLsL, PdBackgroundL, PdInstrReflexAsymmetryL, PdInstrResolutionL,
-    PdMeasL, PdProcL, PdPeakL, Chi2L, RangeL, TextureL, ExcludeL,
-    Pd2dInstrReflexAsymmetryL, Pd2dInstrResolutionL, Pd2dPeakL, DensityPointL,
-    MEMParametersL, SectionL])
+    AtomLocalAxesL, AtomElectronConfigurationL,
+    AtomSiteL, AtomSiteAnisoL, AtomSiteMomentL, AtomSiteScatL,
+    AtomSiteSusceptibilityL, AtomTypeL, AtomTypeScatL,
+    CellL, Chi2L,
+    DensityPointL, DiffrnRadiationL, DiffrnOrientMatrixL, DiffrnReflnL,
+    ExtinctionL, ExcludeL, MEMParametersL,
+    PhaseL, PdBackgroundL, PdInstrReflexAsymmetryL, PdInstrResolutionL,
+    PdMeasL, PdProcL, PdPeakL,
+    Pd2dInstrReflexAsymmetryL, Pd2dInstrResolutionL, Pd2dPeakL,
+    RangeL, RefineLsL, ReflnL, ReflnSusceptibilityL,
+    SectionL, SetupL, SpaceGroupSymopMagnCenteringL,
+    SpaceGroupSymopMagnOperationL, TextureL])
 
-L_DATA_CLASS.extend([Crystal, Diffrn, Pd, Pd2d])
+L_DATA_CLASS.extend([Crystal, MagCrystal, Diffrn, Pd, Pd2d])
 
 L_GLOBAL_CLASS.extend([MEM, RhoChi])
 
@@ -141,11 +157,11 @@ def str_to_items(s_cont: str, item_classes=(), loop_classes=()) -> list:
     data_cif.take_from_string(s_cont)
     l_item_class = L_ITEM_CLASS + list(item_classes)
     l_loop_class = L_LOOP_CLASS + list(loop_classes)
-    
+
     l_item = items_to_itemsn(data_cif.items, l_item_class)
     for loop_cif in data_cif.loops:
-        loop_obj = loop_to_loopn(loop_cif, l_loop_class)
-        l_item.append(loop_obj)
+        l_loop_obj = loop_to_loopn(loop_cif, l_loop_class)
+        l_item.extend(l_loop_obj)
     return l_item
 
 
@@ -169,15 +185,18 @@ def str_to_globaln(s_cont: str, item_classes=(), loop_classes=(),
                 l_cls_global.append(type(item))
 
         for loop_cif in global_cif.loops:
-            loop_obj = loop_to_loopn(loop_cif, l_loop_class)
-            l_global_item.append(loop_obj)
-            if type(loop_obj) not in l_cls_global:
-                l_cls_global.append(type(loop_obj))
+            l_loop_obj = loop_to_loopn(loop_cif, l_loop_class)
+            l_global_item.extend(l_loop_obj)
+            for loop_obj in l_loop_obj:
+                if type(loop_obj) not in l_cls_global:
+                    l_cls_global.append(type(loop_obj))
 
     for data_cif in global_cif.datas:
         flag = False
         str_data = str(data_cif)
         for cls_data in l_data_class:
+            # FIXME: it's bad solution as we loose information which are
+            #        not specified as knonw in cryspy library.
             data_obj = cls_data.from_cif(str_data)
             if data_obj is not None:
                 l_global_item.append(data_obj)
@@ -195,10 +214,11 @@ def str_to_globaln(s_cont: str, item_classes=(), loop_classes=(),
                     l_cls_data.append(type(item))
 
             for loop_cif in data_cif.loops:
-                loop_obj = loop_to_loopn(loop_cif, l_loop_class)
-                l_data_item.append(loop_obj)
-                if type(loop_obj) not in l_cls_data:
-                    l_cls_data.append(type(loop_obj))
+                l_loop_obj = loop_to_loopn(loop_cif, l_loop_class)
+                l_data_item.extend(l_loop_obj)
+                for loop_obj in l_loop_obj:
+                    if type(loop_obj) not in l_cls_data:
+                        l_cls_data.append(type(loop_obj))
 
             if len(l_data_item) > 0:
                 data_obj = items_to_datan(data_cif.name, l_data_item,
@@ -210,20 +230,121 @@ def str_to_globaln(s_cont: str, item_classes=(), loop_classes=(),
     return global_obj
 
 
+def find_name_prefix(loop_name, l_loop_prefix):
+    """Find name prefix."""
+    loop_name = loop_name.strip("_")
+    l_loop = [loop_prefix for loop_prefix in l_loop_prefix if
+              loop_name.startswith(loop_prefix)]
+    if len(l_loop) > 0:
+        l_loop.sort(key=lambda x: len(x))
+        prefix = l_loop[-1]
+        name = loop_name.strip("_")[len(prefix)+1:]
+        separator = loop_name.strip("_")[len(prefix)]
+        return prefix, name, separator
+    return None, None, None
+
+
 # TODO: l_item_class should be added as not necessary l_loop_class fully
 #       correspond to l_item_class
 def loop_to_loopn(loop_cif: Loop, l_loop_class: list) -> LoopN:
-    """Transform Items object to items of ItemN classes."""
-    prefix = loop_cif.prefix
-    flag = False
-    for cls_loop in l_loop_class:
-        if f"_{cls_loop.ITEM_CLASS.PREFIX:}" == prefix:
-            loop_obj = cls_loop.from_cif(str(loop_cif))
-            flag = True
-            break
-    if not(flag):
-        loop_obj = LoopN.from_cif(str(loop_cif))
-    return loop_obj
+    """Transform Loop object to ItemN objects."""
+    # l_item_class for future.
+    l_item_class = []
+    l_item_prefix = [item_cls.PREFIX for item_cls in l_item_class]
+    l_loop_prefix = [loop_cls.ITEM_CLASS.PREFIX for loop_cls in l_loop_class]
+    l_loop_name = [name.lower() for name in loop_cif.names]
+    flag_id = False
+    flag_label = False
+    flag_type = False
+    l_loop_prefix_in = []
+    l_loop_name_in = []
+    l_loop_separator_in = []
+    for loop_name in l_loop_name:
+        prefix, name, separator = find_name_prefix(loop_name,
+                                                   l_loop_prefix)
+        if prefix is not None:
+            l_name_allowed = [h.lower() for h in l_loop_class[
+                l_loop_prefix.index(prefix)].ITEM_CLASS.ATTR_CIF]
+            if not(name.lower() in l_name_allowed):
+                prefix = None
+                name = None
+        if (prefix is None) | (name is None):
+            prefix, name, separator = find_name_prefix(loop_name,
+                                                       l_item_prefix)
+            if prefix is not None:
+                l_name_allowed = [h.lower() for h in l_item_class[
+                    l_item_prefix.index(prefix)].ATTR_CIF]
+                if not(name.lower() in l_name_allowed):
+                    prefix = None
+                    name = None
+        if (prefix is None) | (name is None):
+            if loop_name.find(".") != -1:
+                prefix, name = loop_name.strip("_").split(".")[:2]
+                separator = "."
+            else:
+                prefix = loop_name.strip("_").split("_")[0]
+                name = loop_name.strip("_")[len(prefix)+1:]
+                separator = "_"
+        if name.lower() == "id":
+            flag_id = True
+            prefix_id = prefix
+            loop_name_id = loop_name
+        if name.lower() == "label":
+            flag_label = True
+            prefix_label = prefix
+            loop_name_label = loop_name
+        if name.lower() == "type":
+            flag_type = True
+            prefix_type = prefix
+            loop_name_type = loop_name
+        l_loop_prefix_in.append(prefix)
+        l_loop_name_in.append(name)
+        l_loop_separator_in.append(separator)
+
+    s_loop_prefix_in = set(l_loop_prefix_in)
+    l_loopn = []
+    for prefix in s_loop_prefix_in:
+        item_cls = ItemN
+        loop_cls = LoopN
+        if prefix in l_loop_prefix:
+            loop_cls = l_loop_class[l_loop_prefix.index(prefix)]
+        elif prefix in l_item_prefix:
+            # for future
+            item_cls = l_item_class[l_loop_prefix.index(prefix)]
+        l_loop_name_cif = []
+        ll_val_cif = []
+        separator = "_"
+        for loop_name, prefix_in, name_in, separator_in in zip(
+                l_loop_name, l_loop_prefix_in, l_loop_name_in,
+                l_loop_separator_in):
+            if prefix == prefix_in:
+                l_loop_name_cif.append(loop_name)
+                ll_val_cif.append(loop_cif[loop_name])
+                separator = separator_in
+        if flag_id:
+            if prefix_id != prefix:
+                loop_new_name_id = "_"+prefix+f"{separator:}id"
+                l_loop_name_cif.append(loop_new_name_id)
+                ll_val_cif.append(loop_cif[loop_name_id])
+        elif flag_label:
+            if prefix_label != prefix:
+                loop_new_name_label = "_"+prefix+f"{separator:}label"
+                l_loop_name_cif.append(loop_new_name_label)
+                ll_val_cif.append(loop_cif[loop_name_label])
+        elif flag_type:
+            if prefix_type != prefix:
+                loop_new_name_type = "_"+prefix+f"{separator:}type"
+                l_loop_name_cif.append(loop_new_name_type)
+                ll_val_cif.append(loop_cif[loop_name_type])
+        ll_val_cif_t = [[ll_val_cif[i_2][i_1] for i_2 in range(len(
+            ll_val_cif))] for i_1 in range(len(ll_val_cif[0]))]
+        obj = Loop(names=l_loop_name_cif, values=ll_val_cif_t)
+        loopn = loop_cls.from_cif(str(obj))
+        if not(loopn is None):
+            l_loopn.append(loopn)
+        else:
+            print("l_loop_name_cif: \n", l_loop_name_cif)
+    return l_loopn
 
 
 def items_to_itemsn(items_cif: Items, l_item_class: list) -> List[ItemN]:
@@ -240,15 +361,30 @@ def items_to_itemsn(items_cif: Items, l_item_class: list) -> List[ItemN]:
     for cls_item in l_item_class:
         prefix = cls_item.PREFIX
 
-        l_cls_mand_cif = [f"_{prefix:}{separator:}{attr_cif:}"
+        separator_loc = "_"
+
+        l_cls_mand_cif = [f"_{prefix:}{separator_loc:}{attr_cif:}"
                           for attr_cif in cls_item.ATTR_MANDATORY_CIF]
 
-        l_cls_all_cif = [f"_{prefix:}{separator:}{attr_cif:}"
+        l_cls_all_cif = [f"_{prefix:}{separator_loc:}{attr_cif:}"
                          for attr_cif in cls_item.ATTR_MANDATORY_CIF]
 
         flag_mand = all([items_cif.is_value(name) for name in l_cls_mand_cif])
         flag_any = any([items_cif.is_value(name) for name in l_cls_all_cif])
 
+        if not(flag_any):
+            separator_loc = "."
+            l_cls_mand_cif = [f"_{prefix:}{separator_loc:}{attr_cif:}"
+                              for attr_cif in cls_item.ATTR_MANDATORY_CIF]
+
+            l_cls_all_cif = [f"_{prefix:}{separator_loc:}{attr_cif:}"
+                             for attr_cif in cls_item.ATTR_MANDATORY_CIF]
+
+            flag_mand = all([items_cif.is_value(name) for name in
+                             l_cls_mand_cif])
+            flag_any = any([items_cif.is_value(name) for name in
+                            l_cls_all_cif])
+
         item_obj = None
         if flag_mand:
             items_small = items_cif.items_with_prefix(f"_{prefix:}")
